@@ -1,10 +1,14 @@
+use std::{
+    env, fs,
+    path::{Path, PathBuf},
+};
+
 use anyhow::Context;
 use lazy_static::lazy_static;
-use std::path::{Path, PathBuf};
-use std::{env, fs};
 use tree_sitter::Language;
+use tree_sitter_generate::{ALLOC_HEADER, ARRAY_HEADER};
 use tree_sitter_highlight::HighlightConfiguration;
-use tree_sitter_loader::Loader;
+use tree_sitter_loader::{CompileConfig, Loader};
 use tree_sitter_tags::TagsConfiguration;
 
 include!("./dirs.rs");
@@ -13,7 +17,7 @@ lazy_static! {
     static ref TEST_LOADER: Loader = {
         let mut loader = Loader::with_parser_lib_path(SCRATCH_DIR.clone());
         if env::var("TREE_SITTER_GRAMMAR_DEBUG").is_ok() {
-            loader.use_debug_build(true);
+            loader.debug_build(true);
         }
         loader
     };
@@ -32,13 +36,10 @@ pub fn scratch_dir() -> &'static Path {
 }
 
 pub fn get_language(name: &str) -> Language {
-    TEST_LOADER
-        .load_language_at_path(
-            &GRAMMARS_DIR.join(name).join("src"),
-            &[&HEADER_DIR, &GRAMMARS_DIR.join(name).join("src")],
-            None,
-        )
-        .unwrap()
+    let src_dir = GRAMMARS_DIR.join(name).join("src");
+    let mut config = CompileConfig::new(&src_dir, None, None);
+    config.header_paths.push(&HEADER_DIR);
+    TEST_LOADER.load_language_at_path(config).unwrap()
 }
 
 pub fn get_language_queries_path(language_name: &str) -> PathBuf {
@@ -108,32 +109,16 @@ pub fn get_test_language(name: &str, parser_code: &str, path: Option<&Path>) ->
     let header_path = src_dir.join("tree_sitter");
     fs::create_dir_all(&header_path).unwrap();
 
-    fs::write(header_path.join("alloc.h"), tree_sitter::PARSER_HEADER)
-        .with_context(|| {
-            format!(
-                "Failed to write {:?}",
-                header_path.join("alloc.h").file_name().unwrap()
-            )
-        })
-        .unwrap();
-
-    fs::write(header_path.join("array.h"), tree_sitter::PARSER_HEADER)
-        .with_context(|| {
-            format!(
-                "Failed to write {:?}",
-                header_path.join("array.h").file_name().unwrap()
-            )
-        })
-        .unwrap();
-
-    fs::write(header_path.join("parser.h"), tree_sitter::PARSER_HEADER)
-        .with_context(|| {
-            format!(
-                "Failed to write {:?}",
-                header_path.join("parser.h").file_name().unwrap()
-            )
-        })
-        .unwrap();
+    for (file, content) in [
+        ("alloc.h", ALLOC_HEADER),
+        ("array.h", ARRAY_HEADER),
+        ("parser.h", tree_sitter::PARSER_HEADER),
+    ] {
+        let file = header_path.join(file);
+        fs::write(&file, content)
+            .with_context(|| format!("Failed to write {:?}", file.file_name().unwrap()))
+            .unwrap();
+    }
 
     let paths_to_check = if let Some(scanner_path) = &scanner_path {
         vec![parser_path, scanner_path.clone()]
@@ -141,7 +126,9 @@ pub fn get_test_language(name: &str, parser_code: &str, path: Option<&Path>) ->
         vec![parser_path]
     };
 
-    TEST_LOADER
-        .load_language_at_path_with_name(&src_dir, &[&HEADER_DIR], name, Some(&paths_to_check))
-        .unwrap()
+    let mut config = CompileConfig::new(&src_dir, Some(&paths_to_check), None);
+    config.header_paths = vec![&HEADER_DIR];
+    config.name = name.to_string();
+
+    TEST_LOADER.load_language_at_path_with_name(config).unwrap()
 }
