@@ -17,3 +17,10 @@ mod tree_test;
 
 #[cfg(feature = "wasm")]
 mod wasm_language_test;
+
+pub use crate::fuzz::{
+    allocations,
+    edits::{get_random_edit, invert_edit},
+    random::Rand,
+    ITERATION_COUNT,
+};
