@@ -1,14 +1,12 @@
 use tree_sitter::{Node, Parser, Point, Tree};
+use tree_sitter_generate::{generate_parser_for_grammar, load_grammar_file};
 
 use super::{
     get_random_edit,
     helpers::fixtures::{fixtures_dir, get_language, get_test_language},
     Rand,
 };
-use crate::{
-    generate::{generate_parser_for_grammar, load_grammar_file},
-    parse::perform_edit,
-};
+use crate::parse::perform_edit;
 
 const JSON_EXAMPLE: &str = r#"
 
@@ -171,20 +169,22 @@ fn test_node_child() {
     assert_eq!(tree.root_node().parent(), None);
 
     assert_eq!(
-        tree.root_node()
-            .child_containing_descendant(null_node)
-            .unwrap(),
+        tree.root_node().child_with_descendant(null_node).unwrap(),
         array_node
     );
     assert_eq!(
-        array_node.child_containing_descendant(null_node).unwrap(),
+        array_node.child_with_descendant(null_node).unwrap(),
         object_node
     );
     assert_eq!(
-        object_node.child_containing_descendant(null_node).unwrap(),
+        object_node.child_with_descendant(null_node).unwrap(),
         pair_node
     );
-    assert_eq!(pair_node.child_containing_descendant(null_node), None);
+    assert_eq!(
+        pair_node.child_with_descendant(null_node).unwrap(),
+        null_node
+    );
+    assert_eq!(null_node.child_with_descendant(null_node), None);
 }
 
 #[test]
@@ -286,10 +286,14 @@ fn test_parent_of_zero_width_node() {
     assert_eq!(block_parent.to_string(), "(function_definition name: (identifier) parameters: (parameters (identifier)) body: (block))");
 
     assert_eq!(
-        root.child_containing_descendant(block).unwrap(),
+        root.child_with_descendant(block).unwrap(),
         function_definition
     );
-    assert_eq!(function_definition.child_containing_descendant(block), None);
+    assert_eq!(
+        function_definition.child_with_descendant(block).unwrap(),
+        block
+    );
+    assert_eq!(block.child_with_descendant(block), None);
 
     let code = "<script></script>";
     parser.set_language(&get_language("html")).unwrap();
@@ -318,6 +322,13 @@ fn test_node_field_name_for_child() {
         .child_by_field_name("value")
         .unwrap();
 
+    // -------------------
+    // left: (identifier)  0
+    // operator: "+"       1 <--- (not a named child)
+    // (comment)           2 <--- (is an extra)
+    // right: (identifier) 3
+    // -------------------
+
     assert_eq!(binary_expression_node.field_name_for_child(0), Some("left"));
     assert_eq!(
         binary_expression_node.field_name_for_child(1),
@@ -333,6 +344,44 @@ fn test_node_field_name_for_child() {
     assert_eq!(binary_expression_node.field_name_for_child(4), None);
 }
 
+#[test]
+fn test_node_field_name_for_named_child() {
+    let mut parser = Parser::new();
+    parser.set_language(&get_language("c")).unwrap();
+    let tree = parser
+        .parse("int w = x + /* y is special! */ y;", None)
+        .unwrap();
+    let translation_unit_node = tree.root_node();
+    let declaration_node = translation_unit_node.named_child(0).unwrap();
+
+    let binary_expression_node = declaration_node
+        .child_by_field_name("declarator")
+        .unwrap()
+        .child_by_field_name("value")
+        .unwrap();
+
+    // -------------------
+    // left: (identifier)  0
+    // operator: "+"       _ <--- (not a named child)
+    // (comment)           1 <--- (is an extra)
+    // right: (identifier) 2
+    // -------------------
+
+    assert_eq!(
+        binary_expression_node.field_name_for_named_child(0),
+        Some("left")
+    );
+    // The comment should not have a field name, as it's just an extra
+    assert_eq!(binary_expression_node.field_name_for_named_child(1), None);
+    // The operator is not a named child, so the named child at index 2 is the right child
+    assert_eq!(
+        binary_expression_node.field_name_for_named_child(2),
+        Some("right")
+    );
+    // Negative test - Not a valid child index
+    assert_eq!(binary_expression_node.field_name_for_named_child(3), None);
+}
+
 #[test]
 fn test_node_child_by_field_name_with_extra_hidden_children() {
     let mut parser = Parser::new();
@@ -424,20 +473,22 @@ fn test_node_named_child() {
     assert_eq!(tree.root_node().parent(), None);
 
     assert_eq!(
-        tree.root_node()
-            .child_containing_descendant(null_node)
-            .unwrap(),
+        tree.root_node().child_with_descendant(null_node).unwrap(),
         array_node
     );
     assert_eq!(
-        array_node.child_containing_descendant(null_node).unwrap(),
+        array_node.child_with_descendant(null_node).unwrap(),
         object_node
     );
     assert_eq!(
-        object_node.child_containing_descendant(null_node).unwrap(),
+        object_node.child_with_descendant(null_node).unwrap(),
         pair_node
     );
-    assert_eq!(pair_node.child_containing_descendant(null_node), None);
+    assert_eq!(
+        pair_node.child_with_descendant(null_node).unwrap(),
+        null_node
+    );
+    assert_eq!(null_node.child_with_descendant(null_node), None);
 }
 
 #[test]
