@@ -1,8 +1,10 @@
 use std::{iter, sync::Arc};
 
-use crate::tests::helpers::fixtures::get_language;
+use streaming_iterator::StreamingIterator;
 use tree_sitter::{Language, Node, Parser, Point, Query, QueryCursor, TextProvider, Tree};
 
+use crate::tests::helpers::fixtures::get_language;
+
 fn parse_text(text: impl AsRef<[u8]>) -> (Tree, Language) {
     let language = get_language("c");
     let mut parser = Parser::new();
@@ -29,8 +31,8 @@ fn tree_query<I: AsRef<[u8]>>(tree: &Tree, text: impl TextProvider<I>, language:
     let mut cursor = QueryCursor::new();
     let mut captures = cursor.captures(&query, tree.root_node(), text);
     let (match_, idx) = captures.next().unwrap();
-    let capture = match_.captures[idx];
-    assert_eq!(capture.index as usize, idx);
+    let capture = match_.captures[*idx];
+    assert_eq!(capture.index as usize, *idx);
     assert_eq!("comment", capture.node.kind());
 }
 
