@@ -1,21 +1,8 @@
-use std::path::{Path, PathBuf};
-use std::{env, fs};
+use std::{env, fs, path::PathBuf};
 
 fn main() {
     let out_dir = PathBuf::from(env::var("OUT_DIR").unwrap());
 
-    println!("cargo:rerun-if-env-changed=TREE_SITTER_STATIC_ANALYSIS");
-    if env::var("TREE_SITTER_STATIC_ANALYSIS").is_ok() {
-        if let (Some(clang_path), Some(scan_build_path)) = (which("clang"), which("scan-build")) {
-            let clang_path = clang_path.to_str().unwrap();
-            let scan_build_path = scan_build_path.to_str().unwrap();
-            env::set_var(
-                "CC",
-                format!("{scan_build_path} -analyze-headers --use-analyzer={clang_path} cc",),
-            );
-        }
-    }
-
     #[cfg(feature = "bindgen")]
     generate_bindings(&out_dir);
 
@@ -32,11 +19,10 @@ fn main() {
         config
             .define("TREE_SITTER_FEATURE_WASM", "")
             .define("static_assert(...)", "")
-            .include(env::var("DEP_WASMTIME_C_API_INCLUDE").unwrap())
-            .include(env::var("DEP_WASMTIME_C_API_WASM_INCLUDE").unwrap());
+            .include(env::var("DEP_WASMTIME_C_API_INCLUDE").unwrap());
     }
 
-    let manifest_path = Path::new(env!("CARGO_MANIFEST_DIR"));
+    let manifest_path = PathBuf::from(env::var("CARGO_MANIFEST_DIR").unwrap());
     let include_path = manifest_path.join("include");
     let src_path = manifest_path.join("src");
     let wasm_path = src_path.join("wasm");
@@ -51,9 +37,11 @@ fn main() {
         .flag_if_supported("-fvisibility=hidden")
         .flag_if_supported("-Wshadow")
         .flag_if_supported("-Wno-unused-parameter")
+        .flag_if_supported("-Wno-incompatible-pointer-types")
         .include(&src_path)
         .include(&wasm_path)
         .include(&include_path)
+        .warnings(false)
         .file(src_path.join("lib.c"))
         .compile("tree-sitter");
 
@@ -61,7 +49,7 @@ fn main() {
 }
 
 #[cfg(feature = "bindgen")]
-fn generate_bindings(out_dir: &Path) {
+fn generate_bindings(out_dir: &std::path::Path) {
     const HEADER_PATH: &str = "include/tree_sitter/api.h";
 
     println!("cargo:rerun-if-changed={HEADER_PATH}");
@@ -88,6 +76,7 @@ fn generate_bindings(out_dir: &Path) {
         .allowlist_var("^TREE_SITTER.*")
         .no_copy(no_copy.join("|"))
         .prepend_enum_name(false)
+        .use_core()
         .generate()
         .expect("Failed to generate bindings");
 
@@ -96,16 +85,3 @@ fn generate_bindings(out_dir: &Path) {
         .write_to_file(&bindings_rs)
         .unwrap_or_else(|_| panic!("Failed to write bindings into path: {bindings_rs:?}"));
 }
-
-fn which(exe_name: impl AsRef<Path>) -> Option<PathBuf> {
-    env::var_os("PATH").and_then(|paths| {
-        env::split_paths(&paths).find_map(|dir| {
-            let full_path = dir.join(&exe_name);
-            if full_path.is_file() {
-                Some(full_path)
-            } else {
-                None
-            }
-        })
-    })
-}
